# Sage version information for Python scripts
# This file is auto-generated by the sage-update-version script, do not edit!
version = '9.5.beta7'
date = '2021-11-18'
banner = 'SageMath version 9.5.beta7, Release Date: 2021-11-18'
